class DevRelation(object):
    def __init__(self, devname=None, used=0):
        self.used = used
        self.devname = devname

class Dev(object):
    def __init__(self, devname, size):
        self.devname = devname
        self.size = size

        # list of relations
        self.parents = []
        self.children = []

    def __iadd__(self, o):
        pass

    def get_dev(self, devname):
        return self.tree.get_dev(devname)

    def print_dev(self, level=0, relation=None):
        if relation is None:
            used = self.size
        else:
            used = relation.used
        if self.size == 0:
            pct = 0
        else:
            pct = 100*used//self.size
        s = "%s %d %d %d%%\n" % (self.devname, self.size, used, pct)
        for r in self.children:
            d = self.get_dev(r.devname)
            for i in range(level+1):
                s += " "
            if d is None:
                s += "unknown (dev %s not added)\n"%r.devname
            else:
                s += d.print_dev(level=level+1, relation=r)
        return s

    def get_child(self, devname):
        for r in self.children:
            if r.devname == devname:
                return r
        return None

    def add_child(self, devname, size=0):
        r = self.get_child(devname)
        if r is None:
            r = DevRelation(devname, used=size)
            self.children.append(r)
        else:
            r.used += used
        self.tree.add_dev(devname, size)
        return r

    def is_parent(self, devname):
        for r in self.children:
            if r.devname == devname:
                return True
            d = self.get_dev(r.devname)
            if d.is_parent(devname):
                return True
        return False

class DevTree(object):
    def __init__(self):
        self.dev = {}

        # root node of the relation tree
        self.top = []

    def __iadd__(self, o):
        if isinstance(o, Dev):
            o.tree = self
            self.dev[o.devname] = o
            if not self.has_relations(o.devname):
                r = DevRelation(o.devname)
                self.top.append(o)
        return self

    def __str__(self):
        s = ""
        for r in self.top:
            s += self.dev[r.devname].print_dev()
        return s
        
    def has_relations(self, devname):
        l = []
        for r in self.top:
            if r.devname == devname:
                return True
            d = self.get_dev(r.devname)
            if d.is_parent(devname):
                return True
        return False

    def get_dev(self, devname):
        if devname not in self.dev:
            return None
        return self.dev[devname]

    def add_dev(self, devname, size=0):
        if devname in self.dev:
            return self.dev[devname]
        d = Dev(devname, size)
        self += d
        return d

if __name__ == "__main__":
    tree = DevTree()
    d = tree.add_dev('/dev/sdb', 10000)
    d.add_child('/dev/sdb1', 8000)
    d.add_child('/dev/sdb2', 2000)
    d = tree.add_dev('/dev/sdc', 20000)
    d.add_child('/dev/mapper/vg01-foo', 1000)
    d = tree.get_dev('/dev/sdb2')
    d.add_child('/dev/mapper/vg01-foo', 1000)
    d = tree.get_dev('/dev/mapper/vg01-foo')
    d.add_child('foo.vmdk', 500)
    print tree
